#!/usr/bin/env python
"""
_RunBased_

Run based splitting algorithm that will produce a set of jobs for files in a 
(complete) run. Produces files one run at a time (run:JobGroup) - if the algo
created jobs over more than one run would have confusion in the JobGroup 
creation and/or tracking.

If file spans a run will need to create a mask for that file.
"""

__revision__ = "$Id: RunBased.py,v 1.18 2009/09/30 12:30:54 metson Exp $"
__version__  = "$Revision: 1.18 $"

from sets import Set

from WMCore.JobSplitting.JobFactory import JobFactory
from WMCore.DataStructs.Fileset import Fileset
from WMCore.Services.UUID import makeUUID

class RunBased(JobFactory):
    def algorithm(self, *args, **kwargs):
        """
        _algorithm_
        
        Implement run splitting algorithm. Assumes that if a file with a run is
        present, then all files for that run are also present.
        
        kwargs can take:
        files_per_job - e.g. 20 - Number of files per each split job
        """
        filesPerJob = kwargs.get("files_per_job", 300)
        requireRunClosed = kwargs.get("require_run_closed", False)

        baseName = makeUUID()
                
        # Select all primary files for the first present run
        curRun = None
        primaryFiles = []
        #The current objective of this code is to find all runs in
        #a fileset, and then for each run, create a jobGroup
        #in each jobGroup have a list of jobs containing all the
        #files for that run.
        #If files have more then one run, sort that file with
        #the lowest run
        #In future, mask these files?

        runDict = {}

        locationDict = self.sortByLocation()

        for location in locationDict.keys():
            fileList = locationDict[location]
            for f in fileList:
                
                #If it is a WMBS object, load all data
                if hasattr(f, "loadData"):
                    f.loadData()
                    
                #Die if there are no runs
                if len(f['runs']) < 1:
                    msg = "File %s claims to contain %s runs!" %(f['lfn'], len(f['runs']))
                    raise RuntimeError, msg

                #First we need to pick the lowest run
                run = min(f['runs'])

                #If we don't have the run, we need to add it
                if not run in runDict.keys():
                    runDict[run] = []

                runDict[run].append(f)


            for run in runDict.keys():
                #Find the runs in the dictionary we assembled and split the files in them
            
                self.newGroup()

                #Now split them into sections according to files per job
                while len(runDict[run]) > 0:
                    jobFiles = Fileset()
                    for i in range(filesPerJob):
                        #Watch out if your last job has less then the full number of files
                        if len(runDict[run]) > 0:
                            jobFiles.addFile(runDict[run].pop())

                    # Create the job
                    currentJob = self.newJob('%s-%s' % (baseName, len(self.currentGroup.jobs)), 
                                             files = jobFiles)